@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Parsing of configuration files -
-# Copyright (C) 2013-2016 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -76,7 +76,7 @@ conf_parse_vars_for_each_port_glob_with_bound_val ()
 		pkgsys_eval_ports_glob "$glob_pattern" | while read origin
 		do
 			path=${DBDIR}/conf/each_port/$origin
-			[ -d "$path" ] || mkdir -p "$path"
+			mkdir -p "$path"
 			echo "$val" > $path/${section}.conf
 		done
 	done
@@ -93,8 +93,8 @@ conf_parse_pkgtools_conf ()
 		message_echo "WARNING: portupgrade is not found, pkgtools.conf is ignored" >&2
 		return
 	fi
-	portupgrade_pkg=`pkg_info_qO ports-mgmt/portupgrade`
-	[ -n "$portupgrade_pkg" ] || portupgrade_pkg=`pkg_info_qO ports-mgmt/portupgrade-devel`
+	portupgrade_pkg=`pkgsys_get_installed_pkg_from_origin ports-mgmt/portupgrade`
+	[ -n "$portupgrade_pkg" ] || portupgrade_pkg=`pkgsys_get_installed_pkg_from_origin ports-mgmt/portupgrade-devel`
 	[ `expr "$portupgrade_pkg" : '^portupgrade-devel-'` -eq 0 ] \
 		|| message_echo "WARNING: Combination with portupgrade-devel-* has not tested." >&2
 	sed -n '/^def init_global$/,/^end$/p' "$PORTUPGRADE" > $tmp_script_crop
@@ -188,15 +188,15 @@ eof
 conf_get_complete_var_defs ()
 {
 	case `options_get_effective_opt_load_pkgtoolsconf` in
-	default)
+	default )
 		conf_parse_pkgtools_conf
 		cat "${DBDIR}/conf/pkgtools.parsed" "${CONFFILE}" 2> /dev/null || :
 		;;
-	override)
+	override )
 		conf_parse_pkgtools_conf
 		cat "${CONFFILE}" "${DBDIR}/conf/pkgtools.parsed" 2> /dev/null || :
 		;;
-	no)
+	no )
 		cat "${CONFFILE}" 2> /dev/null || :
 		;;
 	esac
@@ -207,7 +207,7 @@ conf_manipulate_available_var_defs ()
 {
 	(set -e
 		tmpfile_unsetvars=${TMPDIR}/conf_manipulate_available_var_defs::unsetvars.sh
-		_CONFVARS='ENV ALT_MOVED HOLD TABOO NOPKG REPLACE_FROM REPLACE_TO MARG_TARGET MARG_DEF MENV_TARGET MENV_DEF BEFOREBUILD BEFOREDEINSTALL AFTERINSTALL'
+		_CONFVARS='ENV ALT_MOVED HOLD TABOO FREEZE NOPKG REPLACE_FROM REPLACE_TO CONFLICT_TARGET CONFLICT_DEF MARG_TARGET MARG_DEF MENV_TARGET MENV_DEF BEFOREBUILD BEFOREDEINSTALL AFTERINSTALL'
 		for item in ${_CONFVARS}
 		do
 			misc_get_all_vardefs | grep -e "^${item}_" | cut -d = -f 1 | sed 's/^/unset /'
@@ -254,35 +254,34 @@ conf_build_replacement_patterns_from_REPLACE ()
 {
 	local tmp_evaluated_origins
 	tmp_matching_origins=${TMPDIR}/conf_build_replacement_patterns_from_REPLACE:origins
-	cp /dev/null "${DBDIR}/conf/REPLACE.grep_from_pattern"
+	cp /dev/null "${DBDIR}/conf/REPLACE.filter"
 	cp /dev/null "${DBDIR}/conf/REPLACE.sed_pattern"
 	cp /dev/null "${DBDIR}/conf/REPLACE.csv"
 	misc_get_all_vardefs | grep -e '^_CONF_REPLACE_FROM_' | cut -d = -f 1 | while read var
 	do
 		eval glob_pattern=\$\{$var\}
 		eval to=\$\{`echo "$var" | sed 's/^_CONF_REPLACE_FROM_/_CONF_REPLACE_TO_/'`\}
-		pkgsys_eval_ports_glob_even_if_nonexistent "$glob_pattern" > $tmp_matching_origins || :
-		if [ `wc -l < $tmp_matching_origins` -eq 0 ]
+		pkgsys_eval_ports_glob_even_if_nonexistent "$glob_pattern" > ${DBDIR}/conf/REPLACE.filter || :
+		if [ `wc -l < ${DBDIR}/conf/REPLACE.filter` -eq 0 ]
 		then
 			message_echo "WARNING: Original package to be replaced [$glob_pattern] is obsolete." >&2
-			message_echo "If still required, use a pattern for port origins instead." >&2
+			message_echo "If still required, use a pattern for flavored port origins instead." >&2
 			continue
 		fi
-		str_escape_regexp_filter < $tmp_matching_origins \
-			| sed 's|^|^|; s|$|$|' >> ${DBDIR}/conf/REPLACE.grep_from_pattern
 		if [ -z "$to" -o "$to" = delete ]
 		then
 			to=
 		else
-			[ -d "${PORTSDIR}/$to" ] || message_echo "WARNING: replacement port [$to] is obsolete." >&2
+			to_port_path=`pkgsys_get_portpath_from_origin "$to"`
+			[ -d "$to_port_path" ] || message_echo "WARNING: replacement port [$to] is obsolete." >&2
 		fi
-		str_escape_regexp_filter < $tmp_matching_origins \
+		str_escape_regexp_filter < ${DBDIR}/conf/REPLACE.filter \
 			| sed -E "s|^|s:^|; s/$/$:`str_escape_replaceval \"$to\"`:/" \
 			>> ${DBDIR}/conf/REPLACE.sed_pattern
 		while read from
 		do
 			printf '%s\t%s\n' "$from" "$to"
-		done < $tmp_matching_origins >> ${DBDIR}/conf/REPLACE.csv
+		done < ${DBDIR}/conf/REPLACE.filter >> ${DBDIR}/conf/REPLACE.csv
 	done
 }
 
