@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Operations for queries to the temporary database -
-# Copyright (C) 2013-2016 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -16,11 +16,10 @@ database_query_show_list_failure ()
 		message_echo "INFO: No item is registered in this list."
 		return 1
 	fi
-	grep -v -E -f "${DBDIR}/ports_to_delete.grep_pattern" "${DBDIR}/failed.list" \
+	grep -v -E -f "${DBDIR}/ports_to_delete.grep_pattern" "${DBDIR}/failed.list" 2> /dev/null \
 		| while read origin
 	do
-		origin_regexp=`str_escape_regexp "$origin"`
-		grep -q -E "^$origin_regexp$" "${DBDIR}/stage.loop_list/ports_to_delete" 2> /dev/null \
+		grep -q -Fx "$origin" "${DBDIR}/stage.loop_list/ports_to_delete" 2> /dev/null \
 			|| echo "$origin"
 	done > $tmp_valid
 	if [ `cat "$tmp_valid" 2> /dev/null | wc -l` -eq 0 ]
@@ -32,15 +31,13 @@ database_query_show_list_failure ()
 	do
 		note=`cat "${DBDIR}/notes/$origin/note_failtre" 2> /dev/null || :`
 		resolved=no
-		grep -q -E "^`str_escape_regexp \"$origin\"`$" \
-			"${DBDIR}/manually_done.list" 2> /dev/null \
-			&& resolved=yes
-		pkgtag=`cat "${DBDIR}/requires/$origin/pkgtag" 2> /dev/null || :`
+		grep -q -Fx "$origin" "${DBDIR}/manually_done.list" 2> /dev/null && resolved=yes
+		pkgtag=`cat "${DBDIR}/moved_from/$origin/pkgtag" 2> /dev/null || :`
 		if [ $opt_batch_mode = no ]
 		then
 			case $resolved in
-			no)	resolved=;;
-			yes)	resolved=', resolved';;
+			no )	resolved=;;
+			yes )	resolved=', resolved';;
 			esac
 			if [ -n "$note" ]
 			then
@@ -61,7 +58,7 @@ database_query_show_list_failure ()
 	:
 }
 
-# ============= Show a list of failed restoration of conflicts =============
+# ============= Show a list of failed restoration of conflict =============
 database_query_show_list_failed_conflicts_restoration ()
 {
 	if [ `cat "${DBDIR}/deleted_conflicts" 2> /dev/null | wc -l` -eq 0 ]
@@ -73,7 +70,7 @@ database_query_show_list_failed_conflicts_restoration ()
 		| while read origin pkg
 		do
 			pkg_regexp=`str_escape_regexp "$pkg"`
-			against=`grep -E "^$pkg_regexp:" "${DBDIR}/forbidden_conflicts" 2> /dev/null | cut -d : -f 2 | sort -u`
+			against=`grep -E "^$pkg_regexp:" "${DBDIR}/forbidden_conflicts" 2> /dev/null | cut -d : -f 2,3 | sort -u`
 			if [ $opt_batch_mode = no ]
 			then
 				if [ -n "$pkg" ]
@@ -85,28 +82,28 @@ database_query_show_list_failed_conflicts_restoration ()
 				if [ -n "$against" ]
 				then
 					echo -n " against "
-					against=`echo "$against" | tr '\n' ' '`
+					against=`echo "$against" | sed 's/:/(/;s/$/)/' | tr '\n' ' '`
 					str_linearize_list_and "$against"
 				else
 					echo
 				fi
 			else
-				against=`echo "$against" | tr '\n' , | sed 's/,$//'`
+				against=`echo "$against" | tr '\n' '|' | sed 's/,$//'`
 				printf '%s\t%s\t%s\n' "$origin" "$pkg" "$against"
 			fi
 		done
 	:
 }
 
-# ============= Insert initial origins to a list of origins =============
+# ============= Insert initial flavored origins to a list of flavored origins =============
 database_query_add_initial_origins ()
 {
 	local origin
 	while read origin
 	do
 		echo "$origin"
-		[ -e "${DBDIR}/requires/$origin/initial_orig" ] || continue
-		cat "${DBDIR}/requires/$origin/initial_orig"
+		[ -e "${DBDIR}/moved_from/$origin/initial_orig" ] || continue
+		cat "${DBDIR}/moved_from/$origin/initial_orig"
 	done
 	:
 }
@@ -152,20 +149,17 @@ database_query_get_target_attributes ()
 # ============= Check whether (re/de)installation of a port is suppressed =============
 database_query_is_a_port_suppressed ()
 {
-	local origin flag
+	local origin flags flag
 	origin=$1
-	if [ $opt_suppress_self_upadte = yes ]
-	then
-		flag=SUPPRESSED_SELF
-	elif [ $opt_suppress_pkgtools_upadte = yes ]
-	then
-		flag=SUPPRESSED_PKGNG
-	else
-		return 1
-	fi
-	for db in initial requires
+	flags=
+	[ $opt_suppress_self_upadte = yes ] && flags=SUPPRESSED_SELF
+	[ $opt_suppress_pkgtools_upadte = yes ] && flags="$flags SUPPRESSED_PKGNG"
+	for flag in $flags
 	do
-		[ -e "${DBDIR}/$db/$origin/$flag" ] && return
+		for db in initial moved_from
+		do
+			[ -e "${DBDIR}/$db/$origin/$flag" ] && return
+		done
 	done
 	return 1
 }
@@ -173,13 +167,14 @@ database_query_is_a_port_suppressed ()
 # ============= Check whether a port needs to be updated or upgraded =============
 database_query_does_a_port_need_update ()
 {
-	local origin dbpath new_version current_version
+	local origin dbpath frompath new_version current_version
 	origin=$1
 	dbpath=${DBDIR}/requires/$origin
+	frompath=${DBDIR}/moved_from/$origin
 	[ -e "$dbpath/conf_updated" ] && return
 	[ -e "$dbpath/new_version" ] || return
 	new_version=`cat "$dbpath/new_version"`
-	current_version=`cat "$dbpath/current_version"`
+	current_version=`cat "$frompath/current_version"`
 	[ "x$current_version" != "x$new_version" ]
 }
 
@@ -217,7 +212,7 @@ database_query_get_makevar_val ()
 # ============= Check whether configurations for a port is default =============
 database_query_is_default_conf ()
 {
-	local origin mode dbpath tmp_msg is_customized is_requiremnt_replaced files origin_regexp tmp_old tmp_new origin_requirement tmp_msg_customized
+	local origin mode dbpath tmp_msg is_customized is_requiremnt_replaced files tmp_old tmp_new origin_requirement tmp_msg_customized
 	origin=$1
 	mode=$2
 	dbpath=${DBDIR}/requires/$origin
@@ -232,13 +227,12 @@ database_query_is_default_conf ()
 			echo "Knobs and miscellaneous customization by $files," >> $tmp_msg
 			is_customized=yes
 		fi
-		if ! diff "$dbpath/ports_options.default" "$dbpath/ports_options.current" > /dev/null 2>&1
+		if ! diff -q "$dbpath/ports_options.default" "$dbpath/ports_options.current" > /dev/null 2>&1
 		then
 			echo "Non-default port options," >> $tmp_msg
 			is_customized=yes
 		fi
-		origin_regexp=`str_escape_regexp "$origin"`
-		if grep -q -E "^$origin_regexp$" "${DBDIR}/conf/NOPKG:PORTS.parsed" 2> /dev/null
+		if grep -q -Fx "$origin" "${DBDIR}/conf/NOPKG:PORTS.parsed" 2> /dev/null
 		then
 			echo "Explicit specification as non-default in ${APPNAME}.conf," >> $tmp_msg
 			is_customized=yes
@@ -279,7 +273,7 @@ database_query_is_default_conf ()
 }
 
 # ============= Output of lists in which each matching port is registered =============
-database_query_for_list_inclusion_of_matching_port ()
+database_query_show_list_inclusion_of_matching_port ()
 {
 	local grandtitle lists pkgnamedb deptag level isfirst origin_target pkg_target table_target
 	grandtitle=$1
@@ -325,18 +319,16 @@ database_query_for_list_inclusion_of_matching_port ()
 	:
 }
 
-
-# ============= Output of "show" command for each matching port =============
-database_query_for_each_matching_port ()
+# ============= Output of "show" command for port lists =============
+database_query_show_port_lists ()
 {
-	local grandtitle title list pkgnamedb deptag level isfirst origin_target pkg_target table_target list_target
+	local grandtitle title list listdb pkgnamedb isfirst origin_target pkg_target table_target list_target
 	grandtitle=$1
 	title=$2
 	list=$3
-	pkgnamedb=$4
-	deptag=$5
-	level=$6
-	shift 6
+	listdb=$4
+	pkgnamedb=$5
+	shift 5
 	message_echo "[$grandtitle]"
 	message_dependency_scope
 	message_echo
@@ -354,7 +346,7 @@ database_query_for_each_matching_port ()
 		isfirst=n
 		[ $opt_batch_mode = no ] && printf "$title\n" "$origin_target ($pkg_target)"
 		list_target=
-		for table_target in $pkgnamedb
+		for table_target in $listdb
 		do
 			list_target=${DBDIR}/$table_target/$origin_target/$list
 			[ -e "$list_target" ] && break
@@ -395,6 +387,53 @@ database_query_for_each_matching_port ()
 	:
 }
 
+# ============= Output of "show" command for two column lists =============
+database_query_show_two_column_lists ()
+{
+	local grandtitle title list listdb pkgnamedb isfirst origin_target pkg_target table_target list_target
+	grandtitle=$1
+	title=$2
+	list=$3
+	listdb=$4
+	pkgnamedb=$5
+	shift 5
+	message_echo "[$grandtitle]"
+	message_echo
+	isfirst=y
+	for origin_target in `pkgsys_eval_ports_glob "$@"`
+	do
+		pkg_target=
+		for table_target in $pkgnamedb
+		do
+			pkg_target=`cat "${DBDIR}/$table_target/$origin_target/pkgtag" 2> /dev/null` || :
+			[ -n "$pkg_target" ] && break
+		done
+		[ -n "$pkg_target" ] || continue
+		[ "$isfirst" = y ] || message_echo
+		isfirst=n
+		[ $opt_batch_mode = no ] && printf "$title\n" "$origin_target ($pkg_target)"
+		list_target=
+		for table_target in $listdb
+		do
+			list_target=${DBDIR}/$table_target/$origin_target/$list
+			[ -e "$list_target" ] && break
+		done
+		[ -e "$list_target" ] || continue
+		if [ $opt_batch_mode = no ]
+		then
+			sed 's/[[:space:]]/: /' < $list_target
+		else
+			cat  < $list_target
+		fi
+	done
+	if [ "$isfirst" = y ]
+	then
+		message_echo "ERROR: No inspected port matches the glob(s)." >&2
+		exit 1
+	fi
+	:
+}
+
 # ============= Output of "show" command for a single list =============
 database_query_show_single_list ()
 {
@@ -409,17 +448,23 @@ database_query_show_single_list ()
 		message_echo "INFO: No item is registered in this list."
 		return 1
 	fi
+	if ! program_chk_stage_complete PREPARATION
+	then
+		message_echo "WARNING: The temporary database is incomplete. The raw list is shown." >&2
+		cat "${DBDIR}/$list"
+		return
+	fi
 	rm -f "$tmpflag_exists"
 	put_blankline=
 	if [ -n "$flag_filter_only_target" \
 		-a -n "$opt_target_itself$opt_target_dependents$opt_target_requirements" ]
 	then
-		message_echo "INFO: Ports outside of the target scope are excluded."
+		message_echo "WARNING: Ports outside of the target scope are excluded." >&2
 		put_blankline=y
 	fi
 	if [ -n "$flag_filter_skip_unchanged" -a $opt_skip_unchanged = yes ]
 	then
-		message_echo "INFO: Ports which have been the newest with their all requirements from the first are excluded."
+		message_echo "WARNING: Ports which have been the newest with their all requirements from the first are excluded." >&2
 		put_blankline=y
 	fi
 	[ -n "$put_blankline" ] && message_echo
@@ -483,7 +528,7 @@ database_query_show_single_list_exec ()
 	flag_filter_only_target=
 	pkgnamedb='requires obsolete initial'
 	case $subject in
-	todo)
+	todo )
 		message_echo "The following ports remain in the (re)installation queue for the current do/redo process:"
 		message_echo "It is noted that ports to be skipped can be included here."
 		message_dependency_scope
@@ -493,7 +538,7 @@ database_query_show_single_list_exec ()
 		flag_filter_skip_unchanged=necessary_upgrade.$dbsuffix
 		flag_filter_only_target=y
 		;;
-	done)
+	done )
 		message_echo "The following ports have been successfully (re)installed or newly installed:"
 		message_dependency_scope
 		message_echo
@@ -501,7 +546,7 @@ database_query_show_single_list_exec ()
 		flag_filter_skip_unchanged=necessary_upgrade_completed.$dbsuffix
 		flag_filter_only_target=y
 		;;
-	redo)
+	redo )
 		message_echo "The following ports need (re)installation but are to be skipped until any of their failed requirements succeeds:"
 		message_dependency_scope
 		message_echo
@@ -509,46 +554,51 @@ database_query_show_single_list_exec ()
 		flag_filter_skip_unchanged=necessary_upgrade.$dbsuffix
 		flag_filter_only_target=y
 		;;
-	resolved)
+	resolved )
 		message_echo "The following ports had problems which have been manually resolved:"
 		message_echo
 		list=manually_done.list
 		;;
-	failure)
+	failure )
 		message_echo "The following ports experienced failures and kept to be old or uninstalled:"
 		message_echo
 		database_query_show_list_failure
 		return
 		;;
-	conflict)
-		message_echo "The following ports are temporarily deleted due to conflicts:"
+	conflict )
+		message_echo "The following ports are temporarily deleted due to conflict:"
 		message_echo
 		database_query_show_list_failed_conflicts_restoration
 		return
 		;;
-	taboo)
+	taboo )
 		message_echo "The following ports are registered as taboo:"
 		message_echo
 		list=taboo.all.list
 		;;
-	need)
+	freeze )
+		message_echo "The following ports are registered to freeze:"
+		message_echo
+		list=freeze.all.list
+		;;
+	need )
 		message_echo "The following ports are registered as necessary:"
 		message_echo
 		list=need.list
 		;;
-	noneed)
+	noneed )
 		message_echo "The following ports are registered as unnecessary:"
 		message_echo
 		list=noneed.list
 		;;
-	restored)
+	restored )
 		message_echo "The following leaf, obsolete or unneeded ports had been once deleted but are to be or have been restored:"
 		message_echo
 		list=stage.loop_list/ports_to_restore
 		pkgnamedb='obsolete initial'
 		flag_filter_only_target=y
 		;;
-	deleted)
+	deleted )
 		message_echo "The following leaf, obsolete or unneeded ports are to be or have been deleted:"
 		message_echo
 		list=stage.loop_list/ports_to_delete
@@ -563,7 +613,7 @@ database_query_show_single_list_exec ()
 # ============= Check whether a port is registered in a list =============
 database_query_exists_in_list ()
 {
-	local origin subject deptag level tmp_list dbsuffix origin_esc origin_ptn
+	local origin subject deptag level tmp_list dbsuffix origin_esc
 	origin=$1
 	subject=$2
 	deptag=$3
@@ -571,44 +621,87 @@ database_query_exists_in_list ()
 	tmp_list=${TMPDIR}/database_query_exists_in_list:list
 	dbsuffix=$deptag.$level
 	origin_esc=`str_escape_regexp "$origin"`
-	origin_ptn="^$origin_esc$"
 	case $subject in
-	todo)
+	todo )
 		list=stage.loop_list/reinst_todo.remain
 		[ ${DBDIR}/reinst_order.list -nt ${DBDIR}/$list ] && list=reinst_order.list
 		;;
-	done)
+	done )
 		list=success.$dbsuffix.list
 		;;
-	redo)
+	redo )
 		list=todo_after_requirements_succeed.$dbsuffix.list
 		;;
-	resolved)
+	resolved )
 		list=manually_done.list
 		;;
-	failure)
+	failure )
 		list=failed.list
 		;;
-	conflict)
+	conflict )
 		grep -v -E -f "${DBDIR}/ports_to_delete.grep_pattern_col1" "${DBDIR}/deleted_conflicts" 2> /dev/null \
 			| grep -q -E "^${origin_esc}[[:space:]]"
 		return
 		;;
-	taboo)
+	taboo )
 		list=taboo.all.list
 		;;
-	need)
+	freeze )
+		list=freeze.all.list
+		;;
+	need )
 		list=need.list
 		;;
-	noneed)
+	noneed )
 		list=noneed.list
 		;;
-	restored)
+	restored )
 		list=stage.loop_list/ports_to_restore
 		;;
-	deleted)
+	deleted )
 		list=stage.loop_list/ports_to_delete
 		;;
 	esac
-	grep -q -E "$origin_ptn" "${DBDIR}/$list" 2> /dev/null
+	grep -q -Fx "$origin" "${DBDIR}/$list" 2> /dev/null
+}
+
+# ============= Check whether the requirements of installed packages match the port configuration =============
+database_query_dependency_matching ()
+{
+	local origin pkg tmp_inst tmp_db
+	origin=$1
+	[ -d "${DBDIR}/requires/$origin" ] || return
+	tmp_inst=${TMPDIR}/database_query_dependency_matching.installed
+	tmp_db=${TMPDIR}/database_query_dependency_matching.configured
+	pkg=`database_build_get_new_pkgname "$origin"`
+	[ -n "$pkg" ] || return
+	pkg_info_e "$pkg" || return
+	pkg_info_qr "$pkg" | grep -v '^[[:space:]]*$' | sort -u > $tmp_inst
+	database_build_get_full_run_requirement_pkgs "$origin" > $tmp_db
+	diff "$tmp_inst" "$tmp_db" > /dev/null 2>/dev/null
+}
+
+# ============= Check whether any of the requirements are locked because being missing ports to freeze =============
+database_query_are_requirements_not_locked ()
+{
+	local origin
+	origin=$1
+	cat "${DBDIR}/requires/$origin/requirements.all.direct" 2> /dev/null | while read origin_requirement
+	do
+		pkg_requirement=`pkgsys_get_installed_pkg_from_origin "$origin_requirement"` || :
+		if grep -q -Fx "$origin_requirement" "${DBDIR}/freeze.all.list" 2> /dev/null
+		then
+			[ -n "$pkg_requirement" ] || return
+		elif [ -z "$pkg_requirement" ]
+		then
+			database_query_are_requirements_not_locked "$origin_requirement" || return
+		fi
+	done
+	:
+}
+
+# ============= Check whether any of progress is made in the current run =============
+database_query_is_any_progress ()
+{
+	[ `cat "${DBDIR}/new_success_in_current_run" 2> /dev/null | wc -l` -gt 0 ]
 }
