@@ -2,20 +2,17 @@
 # ==============================================================================
 # portsreinstall library script
 # - Operations for recording operation results to the temporary database -
-# Copyright (C) 2013-2016 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
 # ============= Update records by reflecting a successful result of (re/de)installation of a port =============
 database_record_success ()
 {
-	local origin nodedir tmp_grep_failed dbdir_esc tag level dbsuffix
+	local origin nodedir dbdir_esc tag level dbsuffix
 	origin=$1
 	nodedir=${DBDIR}/requires/$origin
-	tmp_grep_failed=${TMPDIR}/database_record_success:grep_failed.list
 	fileedit_add_a_line_if_new "$origin" "${DBDIR}/new_success_in_current_run"
-	cat "${DBDIR}/failed.list" 2> /dev/null | str_escape_regexp_filter \
-		| sed "s|^|^|; s|$|$|" > $tmp_grep_failed || :
 	dbdir_esc=`str_escape_replaceval "${DBDIR}/requires/"`
 	for tag in all run build none
 	do
@@ -36,7 +33,7 @@ database_record_success ()
 				{
 					cat "$nodedir/requirements.$dbsuffix" 2> /dev/null
 					[ $level = full ] && cat "$nodedir/ignored_requirements.$tag" 2> /dev/null
-				} | grep -q -E -f "$tmp_grep_failed"
+				} | grep -q -Fx -f "${DBDIR}/failed.list" 2> /dev/null
 			then
 				fileedit_rm_a_line "$origin" "${DBDIR}/success.$dbsuffix.list"
 				fileedit_add_a_line_if_new "$origin" "${DBDIR}/todo_after_requirements_succeed.$dbsuffix.list"
