@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Messages -
-# Copyright (C) 2013-2016 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -10,15 +10,16 @@
 message_query_yn_default_yes ()
 {
 	local answer
+	[ $opt_batch_mode = yes ] && return 0
 	read answer || :
 	case `echo "$answer" | head -c 1` in
-	y|Y)	message_echo
+	y | Y )	message_echo
 		return 0
 		;;
-	n|N)	message_echo
+	n | N )	message_echo
 		return 1
 		;;
-	'')	return 0
+	'' )	return 0
 		;;
 	*)	message_echo
 		message_echo 'Pardon? ([y]/n)'
@@ -31,15 +32,16 @@ message_query_yn_default_yes ()
 message_query_yn_default_no ()
 {
 	local answer
+	[ $opt_batch_mode = yes ] && return 1
 	read answer || :
 	case `echo "$answer" | head -c 1` in
-	y|Y)	message_echo
+	y | Y )	message_echo
 		return 0
 		;;
-	n|N)	message_echo
+	n | N )	message_echo
 		return 1
 		;;
-	'')	return 1
+	'' )	return 1
 		;;
 	*)	message_echo
 		message_echo 'Pardon? (y/[n])'
@@ -55,10 +57,19 @@ message_version ()
 	echo "${MYVERSION}"
 }
 
+# ============= Output the brief description =============
+message_brief_description ()
+{
+	cat << eof
+ -- Ports upgrading utility for massive forced reinstallation
+ -- And for those who are pursuing the perfect packages environment
+eof
+}
+
 # ============= Output a message except for the batch mode =============
 message_echo ()
 {
-	[ "x$opt_batch_mode" = xyes ] && return
+	[ $opt_batch_mode = yes ] && return
 	echo "$@" | fold -s -w `misc_get_console_column_size`
 }
 
@@ -67,7 +78,7 @@ message_echo ()
 message_cat ()
 {
 	local ncolumns
-	[ "x$opt_batch_mode" = xyes ] && return
+	[ $opt_batch_mode = yes ] && return
 	ncolumns=`misc_get_console_column_size`
 	fold -s -w $ncolumns "$@"
 }
@@ -76,13 +87,12 @@ message_cat ()
 message_credit ()
 {
 	local ncolumns
-	[ "x$opt_batch_mode" = xyes ] && return
+	[ $opt_batch_mode = yes -o "x$opt_no_opening_message" = xyes ] && return
 	ncolumns=`misc_get_console_column_size`
 	message_version | fold -s -w $ncolumns
+	message_brief_description | fold -s -w $ncolumns
 	fold -s -w $ncolumns << eof
- -- Ports upgrading utility for massive forced reinstallation
- -- And for those who are pursuing the perfect packages environment
-Copyright (C) 2010 - 2015 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+Copyright (C) 2010 - 2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 Email: <sakaue.mamoru@samurai.mwghennn.net>
 Homepage: <http://www.mwghennndo.com/software/portsreinstall/>
 eof
@@ -98,7 +108,7 @@ message_timestamp ()
 message_opening_notice ()
 {
 	local ncolumns
-	[ "x$opt_batch_mode" = xyes ] && return
+	[ $opt_batch_mode = yes -o $opt_no_opening_message = yes ] && return
 	ncolumns=`misc_get_console_column_size`
 	fold -s -w $ncolumns << eof
 
@@ -108,6 +118,21 @@ message_opening_notice ()
 eof
 }
 
+# ============= Show option settings =============
+message_show_option_settings ()
+{
+	if [ -e "${DBDIR}/saved_options.sh" \
+		-a $opt_batch_mode = no -a $opt_no_opening_message = no \
+		-a \( $COMMAND_MODE = do -o $COMMAND_MODE = redo \) ]
+	then
+		message_echo "INFO: List of option values:"
+		message_echo "-----------------------------------------"
+		message_cat "${DBDIR}/saved_options.sh"
+		message_echo "-----------------------------------------"
+		message_echo
+	fi
+}
+
 # ============= Report a failure in a process for a port =============
 message_report_failure_for_a_port ()
 {
@@ -120,7 +145,7 @@ message_report_failure_for_a_port ()
 # ============= Output the relation of the current port with specified targets =============
 message_target_relations ()
 {
-	[ "x$opt_batch_mode" = xyes ] && return
+	[ $opt_batch_mode = yes ] && return
 	( set -e
 		origin=$1
 		database_query_get_target_attributes it "$origin"
@@ -160,7 +185,7 @@ message_stage_title ()
 	message_echo "[At `message_timestamp`]"
 }
 
-# ============= Dummy output for dry run =============
+# ============= Message of dry run =============
 message_dry_run ()
 {
 	message_echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
@@ -169,6 +194,15 @@ message_dry_run ()
 	message_echo
 }
 
+# ============= Message of fetch-only mode =============
+message_fetch_only ()
+{
+	message_echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
+	message_echo "@@@@@@@@@@@@@ FETCH-ONLY MODE @@@@@@@@@@@@@"
+	message_echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
+	message_echo
+}
+
 # ============= Output a summary of a resource recorded in a list =============
 message_summary_dependents_of_failed_reinstallation ()
 {
@@ -239,12 +273,9 @@ eof
 # ============= Output a warning if no progress was achieved =============
 message_warn_no_achieved_progress ()
 {
-	[ `cat "${DBDIR}/new_success_in_current_run" 2> /dev/null | wc -l` -gt 0 ] && return
-	[ $opt_batch_mode = yes ] && return 1
+	[ $opt_batch_mode = yes ] && return
 	message_echo '****************'
-	message_echo "WARNING: No progress was achieved by this run."
-	message_echo "         Manual solution is needed on the failed ports."
+	message_echo "WARNING: Some problems remain. Manual troubleshooting is required."
 	message_echo '****************'
 	message_echo
-	return 1
 }
