@@ -1,8 +1,8 @@
 #!/bin/sh -e
 # ==============================================================================
 # portsreinstall library script
-# - Command line options -
-# Copyright (C) 2013-2016 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# - Interface of libraries for command line options -
+# Copyright (C) 2013-2018 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -17,15 +17,6 @@ OPTIONS_SHIFT=0
 options_db_onetime ()
 {
 	cat << eof
-h	help	opt_help_mode	0	1
-h	short-help	opt_help_mode	0	1
-H	long-help	opt_help_mode	0	2
-V	show-version	opt_show_version	no	yes
-a	batch-mode	opt_batch_mode	no	yes
-N	reset-targets	opt_reset_targets	no	yes
-M	reset-minor-options	opt_reset_minor_options	no	yes
-L	reload-conf	opt_reload_conf	no	yes
-i	allow-new-targets	opt_allow_new_targets	no	yes
 eof
 }
 
@@ -33,24 +24,6 @@ eof
 options_db_saved_and_renewable ()
 {
 	cat << eof
-C	apply-default-config	opt_apply_default_config	no	yes
-A	non-interactive-ports-only	opt_batch_ports_only	no	yes
-I	interactive-ports-only	opt_interactive_ports_only	no	yes
-s	avoid-vulnerability-check	opt_avoid_vulner	no	yes
-q	skip-unchanged-ports	opt_skip_unchanged	no	yes
-b	include-buildtime-dependencies	opt_include_buildtime_dependencies	no	yes
-B	exclude-runtime-dependencies	opt_include_runtime_dependencies	yes	no
-d	keep-distfiles	opt_keep_distfiles	no	yes
-k	suppress-self-upadte	opt_suppress_self_upadte	no	yes
-g	suppress-pkgtools-upadte	opt_suppress_pkgtools_upadte	no	yes
-c	suppress-clean-obsolete-database	opt_suppress_obsolete_db_clean	no	yes
-D	suppress-entire-inspection-distinfo	opt_inspect_entire_distinfo	yes	no
-n	dry-run	opt_dry_run	no	yes
-X	deselect-all	opt_dialog_auto	no	none
-Y	select-all	opt_dialog_auto	no	all
-G	use-prebuilt-package	opt_inst_by_pkg_if_can	no	yes
-l	use-legacy-package-for-missing-pkgng	opt_use_legacy_pkg_for_missing_pkgng	no	yes
-f	disallow-force-continuation-for-looped-dependency	opt_disallow_force_continuation_for_looped_dependency	no	yes
 eof
 }
 
@@ -58,10 +31,6 @@ eof
 options_db_saved_and_renewable_in_redo_on_target ()
 {
 	cat << eof
-t	target-and-dependents	opt_target_dependents	''	:
-T	target-and-requirements	opt_target_requirements	''	:
-O	target-only-itself	opt_target_itself	''	:
-o	only-target-scope	opt_only_target_scope	no	yes
 eof
 }
 
@@ -69,9 +38,6 @@ eof
 options_db_saved_and_renewable_in_redo_on_conf ()
 {
 	cat << eof
-p	load-pkgtoolsconf-as-default	opt_load_pkgtoolsconf	undef	default
-P	load-pkgtoolsconf-as-override	opt_load_pkgtoolsconf	undef	override
-Q	ignore-pkgtoolsconf	opt_load_pkgtoolsconf	undef	no
 eof
 }
 
@@ -82,6 +48,11 @@ options_db_saved_and_non_renewable ()
 eof
 }
 
+# ============= Regularize the option value =============
+options_regularize ()
+{
+}
+
 # ============= Database of all options to be saved =============
 options_db_saved ()
 {
@@ -106,20 +77,27 @@ options_db ()
 		while [ $# -gt 0 ]
 		do
 			case $1 in
-			renewable_anytime)
-				options_db_saved_and_renewable;;
-			renewable_in_redo_on_target)
-				options_db_saved_and_renewable_in_redo_on_target;;
-			renewable_in_redo_on_conf)
-				options_db_saved_and_renewable_in_redo_on_conf;;
-			non_renewable)
-				options_db_saved_and_non_renewable;;
-			saved)
-				options_db_saved;;
-			onetime)
-				options_db_onetime;;
-			all)
-				options_db_all;;
+			renewable_anytime )
+				options_db_saved_and_renewable
+				;;
+			renewable_in_redo_on_target )
+				options_db_saved_and_renewable_in_redo_on_target
+				;;
+			renewable_in_redo_on_conf )
+				options_db_saved_and_renewable_in_redo_on_conf
+				;;
+			non_renewable )
+				options_db_saved_and_non_renewable
+				;;
+			saved )
+				options_db_saved
+				;;
+			onetime )
+				options_db_onetime
+				;;
+			all )
+				options_db_all
+				;;
 			*)
 				message_echo "ERROR: Invalid argument [$1] for options_inverse_parse" >&2
 				exit 1;;
@@ -338,10 +316,9 @@ options_select_new_ports_if_duplicated ()
 		cp /dev/null "$tmp_new_ports"
 		while read origin
 		do
-			origin_regexp=`str_escape_regexp "$origin"`
-			if pkgsys_pkg_info_eO "$origin" \
+			if pkgsys_exists_or_existed_from_orig "$origin" \
 				|| cat "${DBDIR}/installed_ports" "${DBDIR}/targets_specified_so_far" 2> /dev/null \
-					| grep -q -E "$origin_regexp"
+					| grep -q -Fx "$origin"
 			then
 				echo "$origin" >> $tmp_existing_ports
 			else
@@ -380,14 +357,18 @@ options_select_new_ports_if_duplicated ()
 options_get_dependency_type ()
 {
 	case $opt_include_buildtime_dependencies+$opt_include_runtime_dependencies in
-	yes+yes)
-		echo all;;
-	no+yes)
-		echo run;;
-	yes+no)
-		echo build;;
-	no+no)
-		echo none;;
+	yes+yes )
+		echo all
+		;;
+	no+yes )
+		echo run
+		;;
+	yes+no )
+		echo build
+		;;
+	no+no )
+		echo none
+		;;
 	esac
 }
 
@@ -397,19 +378,19 @@ options_parse_dependency_type ()
 	local deptag
 	deptag=$1
 	case $deptag in
-	all)
+	all )
 		opt_include_buildtime_dependencies=yes
 		opt_include_runtime_dependencies=yes
 		;;
-	run)
+	run )
 		opt_include_buildtime_dependencies=no
 		opt_include_runtime_dependencies=yes
 		;;
-	build)
+	build )
 		opt_include_buildtime_dependencies=yes
 		opt_include_runtime_dependencies=no
 		;;
-	none)
+	none )
 		opt_include_buildtime_dependencies=no
 		opt_include_runtime_dependencies=no
 		;;
@@ -420,10 +401,12 @@ options_parse_dependency_type ()
 options_get_dependency_level ()
 {
 	case $opt_only_target_scope in
-	yes)
-		echo direct;;
-	no)
-		echo full;;
+	yes )
+		echo direct
+		;;
+	no )
+		echo full
+		;;
 	esac
 }
 
@@ -433,10 +416,12 @@ options_parse_dependency_level ()
 	local level
 	level=$1
 	case $level in
-	direct)
-		opt_only_target_scope=yes;;
-	full)
-		opt_only_target_scope=no;;
+	direct )
+		opt_only_target_scope=yes
+		;;
+	full )
+		opt_only_target_scope=no
+		;;
 	esac
 }
 
@@ -446,14 +431,18 @@ options_get_dependency_msgterm ()
 	local scope
 	scope=`options_get_dependency_level`
 	case $opt_include_buildtime_dependencies+$opt_include_runtime_dependencies in
-	yes+yes)
-		echo "$scope run- and build-time";;
-	no+yes)
-		echo "$scope run-time";;
-	yes+no)
-		echo "$scope build-time";;
-	no+no)
-		echo 'no';;
+	yes+yes )
+		echo "$scope run- and build-time"
+		;;
+	no+yes )
+		echo "$scope run-time"
+		;;
+	yes+no )
+		echo "$scope build-time"
+		;;
+	no+no )
+		echo 'no'
+		;;
 	esac
 }
 
@@ -477,3 +466,54 @@ options_get_effective_opt_load_pkgtoolsconf ()
 	echo "$effective_opt_load_pkgtoolsconf"
 }
 
+# ============= Show the setting of an option group =============
+options_show_group_setting ()
+{
+	local opt_control group_reset group_remain reset_opts result_opts str_opt_control str_rstoption notice_reset notice_result
+	opt_control=$1
+	group_reset=$2
+	group_remain=$3
+	reset_opts=`options_inverse_parse $group_reset`
+	result_opts=`options_inverse_parse $group_remain`
+	if [ $is_batch_mode = yes ]
+	then
+		printf '%s\t%s\t%s\n' "$opt_control" "$reset_opts" "$result_opts"
+	else
+		str_opt_control=`echo "$opt_control" | sed -E 's/(.)/-\1 /g;s/ *$//'`
+		str_rstoption=Option`[ \`echo -n "$opt_control" | wc -c\` -ge 2 ] && echo s` || :
+		notice_reset=`[ -z "$reset_opts" ] && echo ' (nothing)'` || :
+		notice_result=`[ -z "$result_opts" ] && echo ' (default)'` || :
+		echo "$str_rstoption [$str_opt_control] will reset [$reset_opts]$notice_reset so that the settings become [$result_opts]$notice_result."
+	fi
+}
+
+# ============= Show the all options =============
+options_show_all ()
+{
+		( set -e
+			is_batch_mode=$opt_batch_mode
+			options_set_default
+			[ -e "${DBDIR}/saved_options.sh" ] && . "${DBDIR}/saved_options.sh"
+			savedopts=`options_inverse_parse`
+			if [ $is_batch_mode = yes ]
+			then
+				printf '\t%s\n' "$savedopts"
+			else
+				echo "The saved setting is [$savedopts]."
+			fi
+			options_show_group_setting M \
+				'renewable_anytime' 'non_renewable renewable_in_redo_on_target renewable_in_redo_on_conf'
+			options_show_group_setting N \
+				'renewable_in_redo_on_target' 'non_renewable renewable_anytime renewable_in_redo_on_conf'
+			options_show_group_setting L \
+				'renewable_in_redo_on_conf' 'non_renewable renewable_anytime renewable_in_redo_on_target'
+			options_show_group_setting MN \
+				'renewable_anytime renewable_in_redo_on_target' 'non_renewable renewable_in_redo_on_conf'
+			options_show_group_setting ML \
+				'renewable_anytime renewable_in_redo_on_conf' 'non_renewable renewable_in_redo_on_target'
+			options_show_group_setting NL \
+				'renewable_in_redo_on_target renewable_in_redo_on_conf' 'non_renewable renewable_anytime'
+			options_show_group_setting MNL \
+				'renewable_anytime renewable_in_redo_on_target renewable_in_redo_on_conf' 'non_renewable'
+		)
+}
